      MODULE globalmod_constants
          implicit none
          integer*4, parameter           ::  idread = 5, idwrite = 6
          character(LEN=50)              ::  machine_name
          integer*4                      ::  version,subversion
          integer, parameter             ::  idp = 8,master_id = 0
          integer*4                      ::  my_id,numprocs

          integer*8                      ::  nsize,chunk,istart,iend
      END MODULE globalmod_constants


!=================================================================!
! Calculate \pi.                                                  !
!-----------------------------------------------------------------!
! (1) Pure MPI version.                                           !
! (2) MPI_REDUCE().                                               !
!-----------------------------------------------------------------!
! Xiaoxu Guan at LSU HPC                                          !
! May 10, 2017                                                    !
!=================================================================!
      program mpi_openmp_pi
      use globalmod_constants
      implicit none
      include 'mpif.h'
      integer                 :: ierr,my_len
      integer*4               :: itemp,i,j,k,nmt,istart_min,iend_max
      real(KIND=idp)          :: time_s,time_e,temp
      character(LEN=20)       :: message
      integer*4               :: istatus(MPI_STATUS_SIZE)
      integer                 :: argc            ! Number of command line arguments.
      integer                 :: iargc           ! External function returning argc.
      character(LEN=80)       :: arg             ! Container for command line arguments.
      integer*4               :: chunk_floor,chunk_ceili,rmn
      real(KIND=idp)          :: xi,tmp,fsum,pi,h


      call MPI_INIT( ierr )
      call MPI_COMM_SIZE( MPI_COMM_WORLD, numprocs, ierr )
      call MPI_COMM_RANK( MPI_COMM_WORLD, my_id, ierr )

      call MPI_GET_PROCESSOR_NAME( machine_name, my_len, ierr )
      call MPI_GET_VERSION( version, subversion, ierr )


      if(my_id == master_id ) then
         message = 'Job submitted on :: '
         call datetime(message)
         time_s = MPI_WTIME()
      end if

      call MPI_BARRIER( MPI_COMM_WORLD,ierr )

      argc=iargc()
      if (argc /= 1) then ! number of input arguments must be 1.

          if (my_id == master_id) then
              write(idwrite,'(1x,a,i3)') 'Number of input arguments is ', argc
              write(idwrite,'(1x,a)')    'It must be 1.'
              write(idwrite,'(1x,a)')    'Quit!'
          end if
              call MPI_FINALIZE(ierr)
              stop
       else    ! passing the arguments.

          call getarg(1, arg)
          read(arg, '(i)') nsize   ! number of mesh points.

          if(my_id == master_id) then
             write(idwrite,'(1x,a,i14)')  'Number of points: ', nsize
             write(idwrite,*)
          end if
       end if

      call MPI_BARRIER( MPI_COMM_WORLD,ierr )

! domain decomposition.
      temp = dble(nsize) / dble(numprocs)
      chunk_floor = floor(temp) 
      chunk_ceili = ceiling(temp) 

      rmn = mod(nsize,numprocs) 

! determine the start and end indices: istar and iend.
      if( rmn > 0 .and. my_id <= rmn-1 ) then
          chunk  = chunk_ceili
          istart = my_id * chunk + 1
          iend   = istart + chunk - 1
      elseif( rmn > 0 .and. my_id > rmn-1 ) then
          chunk  = chunk_floor
          itemp  = chunk_ceili * rmn 
          istart = itemp + 1 + (my_id - rmn ) * chunk
          iend   = istart + chunk - 1
      elseif(rmn == 0) then
          chunk = chunk_floor
          istart = my_id * chunk + 1
          iend   = istart + chunk - 1
      end if

      write(idwrite,'(1x,5(i14,3x))')  my_id, istart,iend, chunk, nsize

      call MPI_BARRIER( MPI_COMM_WORLD,ierr )

! sum over points.
         h = 1.0_idp / dble(nsize)
         fsum = 0.0_idp
      do i = istart, iend
         xi = h * (dble(i)-0.5_idp)
         tmp = 1.0_idp + xi * xi
         fsum = fsum + 1.0_idp / tmp
      end do

      fsum = 4.0_idp * h * fsum
      call MPI_REDUCE(fsum,pi,1,MPI_DOUBLE_PRECISION,MPI_SUM,0,MPI_COMM_WORLD,ierr)

      if(my_id == master_id ) then
         time_e = MPI_WTIME()
         write(idwrite,*)
         write(idwrite,'(1x,f18.15)') pi 
         write(idwrite,*)
         write(idwrite,*)
         write(idwrite,'(1x,a,f10.5)') 'Elapsed time (sec) =  ', time_e - time_s
         message = 'Job finished on  :: '
         call datetime(message)
      end if


      call MPI_FINALIZE(ierr)

      end program mpi_openmp_pi



!===========================================================!
! This subroutine reads the input parameters and generate   !
! the mesh points of xi, eta, and other global variables.   !
! 'ier_read' returns the error code.                        !
!  ier_read = 0 for normal return without errors.           !
! Otherwise, it takes a non-zero integer.                   !
!===========================================================!
      subroutine datetime(message)
      use globalmod_constants, only  : idwrite
      implicit none
      character(LEN=20), intent(in) :: message
      character(LEN=8)              :: today
      character(LEN=10)             :: now


      call date_and_time (today, now)
! today: mm/dd/yyyy
      write(idwrite,'(1x,a,a2,a,a2,a,a4,a)') message, today(5:6),        &
                                     '/', today(7:8), '/', today(1:4),   &
                                     '  (/mm/dd/yyyy)'
! now: hhmmss.sss
      write(idwrite,'(1x,a,13x,a,a2,a,a2,a,a5)') 'Time',':: ',now(1:2), ':',    &
                                                           now(3:4), ':', now(5:10)
      write(idwrite,*)

      return
      end subroutine datetime
